<?php
namespace App\Entity;

use Doctrine\ORM\Mapping as ORM;
use Doctrine\Common\Collections\ArrayCollection;

use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * @ORM\Entity
 * @ORM\Table(name="groupe")
 * @ORM\HasLifecycleCallbacks()
 * @ORM\Entity(repositoryClass="App\Repository\GroupRepository")
 * 
 * @UniqueEntity(fields="label", message="Un group existe déjà avec ce label")
 */
class Group
{
    /**
     * @ORM\Column(type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @ORM\Column(type="string", length=250, unique=true)
     */
    private $label;

    /**
    * @var string
    *
    * @ORM\Column(name="description", type="text", nullable=true)
    */
    private $description;

    /**
     * @var string
     *
     * @ORM\Column(name="color", type="string", length=24,  nullable=true)
     */
    private $color;

    /**
     * @ORM\Column(type="string", length=250, nullable=true)
     */
    private $email;

    /**
     * @ORM\Column(type="boolean", options={"default" : false})
     */
    private $fgopen;
    
    /**
     * @ORM\Column(type="boolean", options={"default" : true})
     */
    private $fgcanshare;    

    /**
     * @ORM\Column(type="boolean", options={"default" : false})
     */
    private $fgcancreatepage;  

    /**
     * @ORM\Column(type="boolean", options={"default" : false})
     */
    private $fgcancreateblog;  

    /**
     * @ORM\Column(type="boolean", options={"default" : false})
     */
    private $fgcancreatecalendar;  

    /**
     * @ORM\Column(type="boolean", options={"default" : false})
     */
    private $fgcancreateproject;  

    /**
     * @ORM\Column(type="boolean", options={"default" : false})
     */
    private $fgall;

    /**
     * @ORM\Column(type="boolean", options={"default" : false})
     */
    private $fgtemplate;    

    /**
     * @ORM\Column(type="text", nullable=true)
     */
    private $ldapfilter;

    /**
     * @ORM\Column(type="text", nullable=true)
     */
    private $attributes;

    /**
     * @ORM\Column(type="string", length=250, nullable=true)
     */
    private $idonlyoffice;

    /**
     * @ORM\Column(name="invitations", type="array", nullable=true)
     */
    private $invitations;

    /**
     * @ORM\ManyToOne(targetEntity="Icon", inversedBy="groups")
     * @ORM\JoinColumn(nullable=true, onDelete="SET NULL")
     */
    private $icon;

    /**
     * @ORM\ManyToOne(targetEntity="User", inversedBy="ownergroups")
     * @ORM\JoinColumn(nullable=true, onDelete="SET NULL")
     */
    private $owner;

    /**
     * @ORM\ManyToOne(targetEntity="Page", inversedBy="templategroups")
     */
    private $pagetemplate;

    /**
     * @var ArrayCollection $users
     * @var UserGroup
     *
     * @ORM\OneToMany(targetEntity="UserGroup", mappedBy="group", cascade={"persist"}, orphanRemoval=true)
     */
    private $users;

    /**
    * @ORM\ManyToMany(targetEntity="Item", mappedBy="groups")
    */    
    protected $items;

    /**
    * @ORM\ManyToMany(targetEntity="Alert", mappedBy="groups")
    */    
    protected $alerts;

    /**
    * @ORM\ManyToMany(targetEntity="Page", mappedBy="groups")
    */    
    protected $pages;

    /**
    * @ORM\ManyToMany(targetEntity="Flux", mappedBy="groups")
    */    
    protected $fluxs;

    /**
    * @ORM\ManyToMany(targetEntity="Notice", mappedBy="groups")
    */    
    protected $notices;    

    /**
    * @ORM\ManyToMany(targetEntity="Calendar", mappedBy="groups")
    */    
    protected $calendars;  

    /**
    * @ORM\ManyToMany(targetEntity="Calendarevent", mappedBy="groups")
    */    
    protected $calendarevents;  

    /**
    * @ORM\ManyToMany(targetEntity="Blog", mappedBy="groups")
    */    
    protected $blogs;  

    /**
    * @ORM\ManyToMany(targetEntity="Project", mappedBy="groups")
    */    
    protected $projects;  
    
    /**
     * @var ArrayCollection $message
     * @var Message
     *
     * @ORM\OneToMany(targetEntity="Message", mappedBy="group", cascade={"persist"}, orphanRemoval=true)
     */
    protected $messages;  

    /**
     * @ORM\OneToMany(targetEntity="Statistic", mappedBy="group", cascade={"persist"}, orphanRemoval=true)
     */
    private $statistics;

    // Champs temporaire
    protected $nosynconly;
    public function getNosynconly()
    {
        return $this->nosynconly;
    }
    public function setNosynconly($nosynconly)
    {
        $this->nosynconly = $nosynconly;
        return $this;
    }



    // ATTENTION NE SURTOUT PAS REGEN TOTALEMENT LES SETTER
    // Methode custom sur les Addxxx et Removexxx

    /**
     * Constructor
     */
    public function __construct()
    {
        $this->users = new \Doctrine\Common\Collections\ArrayCollection();
        $this->items = new \Doctrine\Common\Collections\ArrayCollection();
        $this->alerts = new \Doctrine\Common\Collections\ArrayCollection();
        $this->pages = new \Doctrine\Common\Collections\ArrayCollection();
        $this->fluxs = new \Doctrine\Common\Collections\ArrayCollection();
        $this->notices = new \Doctrine\Common\Collections\ArrayCollection();
        $this->calendars = new \Doctrine\Common\Collections\ArrayCollection();
        $this->blogs = new \Doctrine\Common\Collections\ArrayCollection();
        $this->projects = new \Doctrine\Common\Collections\ArrayCollection();
    }

    /**
     * Get id
     *
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set label
     *
     * @param string $label
     *
     * @return Group
     */
    public function setLabel($label)
    {
        $this->label = $label;

        return $this;
    }

    /**
     * Get label
     *
     * @return string
     */
    public function getLabel()
    {
        return $this->label;
    }

    /**
     * Set fgopen
     *
     * @param boolean $fgopen
     *
     * @return Group
     */
    public function setFgopen($fgopen)
    {
        $this->fgopen = $fgopen;

        return $this;
    }

    /**
     * Get fgopen
     *
     * @return boolean
     */
    public function getFgopen()
    {
        return $this->fgopen;
    }

    /**
     * Set fgcanshare
     *
     * @param boolean $fgcanshare
     *
     * @return Group
     */
    public function setFgcanshare($fgcanshare)
    {
        $this->fgcanshare = $fgcanshare;

        return $this;
    }

    /**
     * Get fgcanshare
     *
     * @return boolean
     */
    public function getFgcanshare()
    {
        return $this->fgcanshare;
    }

    /**
     * Set fgcancreatepage
     *
     * @param boolean $fgcancreatepage
     *
     * @return Group
     */
    public function setFgcancreatepage($fgcancreatepage)
    {
        $this->fgcancreatepage = $fgcancreatepage;

        return $this;
    }

    /**
     * Get fgcancreatepage
     *
     * @return boolean
     */
    public function getFgcancreatepage()
    {
        return $this->fgcancreatepage;
    }

    /**
     * Set fgcancreateblog
     *
     * @param boolean $fgcancreateblog
     *
     * @return Group
     */
    public function setFgcancreateblog($fgcancreateblog)
    {
        $this->fgcancreateblog = $fgcancreateblog;

        return $this;
    }

    /**
     * Get fgcancreateblog
     *
     * @return boolean
     */
    public function getFgcancreateblog()
    {
        return $this->fgcancreateblog;
    }

    /**
     * Set fgcancreatecalendar
     *
     * @param boolean $fgcancreatecalendar
     *
     * @return Group
     */
    public function setFgcancreatecalendar($fgcancreatecalendar)
    {
        $this->fgcancreatecalendar = $fgcancreatecalendar;

        return $this;
    }

    /**
     * Get fgcancreatecalendar
     *
     * @return boolean
     */
    public function getFgcancreatecalendar()
    {
        return $this->fgcancreatecalendar;
    }

    /**
     * Set fgall
     *
     * @param boolean $fgall
     *
     * @return Group
     */
    public function setFgall($fgall)
    {
        $this->fgall = $fgall;

        return $this;
    }

    /**
     * Get fgall
     *
     * @return boolean
     */
    public function getFgall()
    {
        return $this->fgall;
    }

    /**
     * Set fgtemplate
     *
     * @param boolean $fgtemplate
     *
     * @return Group
     */
    public function setFgtemplate($fgtemplate)
    {
        $this->fgtemplate = $fgtemplate;

        return $this;
    }

    /**
     * Get fgtemplate
     *
     * @return boolean
     */
    public function getFgtemplate()
    {
        return $this->fgtemplate;
    }

    /**
     * Set ldapfilter
     *
     * @param string $ldapfilter
     *
     * @return Group
     */
    public function setLdapfilter($ldapfilter)
    {
        $this->ldapfilter = $ldapfilter;

        return $this;
    }

    /**
     * Get ldapfilter
     *
     * @return string
     */
    public function getLdapfilter()
    {
        return $this->ldapfilter;
    }

    /**
     * Set attributes
     *
     * @param string $attributes
     *
     * @return Group
     */
    public function setAttributes($attributes)
    {
        $this->attributes = $attributes;

        return $this;
    }

    /**
     * Get attributes
     *
     * @return string
     */
    public function getAttributes()
    {
        return $this->attributes;
    }

    /**
     * Set idonlyoffice
     *
     * @param string $idonlyoffice
     *
     * @return Group
     */
    public function setIdonlyoffice($idonlyoffice)
    {
        $this->idonlyoffice = $idonlyoffice;

        return $this;
    }

    /**
     * Get idonlyoffice
     *
     * @return string
     */
    public function getIdonlyoffice()
    {
        return $this->idonlyoffice;
    }

    /**
     * Add user
     *
     * @param UserGroup $user
     *
     * @return Group
     */
    public function addUser(UserGroup $user)
    {
        $this->users[] = $user;

        return $this;
    }

    /**
     * Remove user
     *
     * @param UserGroup $user
     */
    public function removeUser(UserGroup $user)
    {
        $this->users->removeElement($user);
    }

    /**
     * Get users
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getUsers()
    {
        return $this->users;
    }

    /**
     * Add item
     *
     * @param Item $item
     *
     * @return Group
     */
    public function addItem(Item $item)
    {
        $item->addGroup($this);
        $this->items[] = $item;

        return $this;
    }

    /**
     * Remove item
     *
     * @param Item $item
     */
    public function removeItem(Item $item)
    {
        $item->removeGroup($this);
        $this->items->removeElement($item);
    }

    /**
     * Get items
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getItems()
    {
        return $this->items;
    }

    /**
     * Add alert
     *
     * @param Alert $alert
     *
     * @return Group
     */
    public function addAlert(Alert $alert)
    {
        $alert->addGroup($this);
        $this->alerts[] = $alert;

        return $this;
    }

    /**
     * Remove alert
     *
     * @param Alert $alert
     */
    public function removeAlert(Alert $alert)
    {
        $alert->removeGroup($this);
        $this->alerts->removeElement($alert);
    }

    /**
     * Get alerts
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getAlerts()
    {
        return $this->alerts;
    }

    /**
     * Add page
     *
     * @param Page $page
     *
     * @return Group
     */
    public function addPage(Page $page)
    {
        $page->addGroup($this);
        $this->pages[] = $page;

        return $this;
    }

    /**
     * Remove page
     *
     * @param Page $page
     */
    public function removePage(Page $page)
    {
        $page->removeGroup($this);
        $this->pages->removeElement($page);
    }

    /**
     * Get pages
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getPages()
    {
        return $this->pages;
    }

    /**
     * Add flux
     *
     * @param Flux $flux
     *
     * @return Group
     */
    public function addFlux(Flux $flux)
    {
        $flux->addGroup($this);
        $this->fluxs[] = $flux;

        return $this;
    }

    /**
     * Remove flux
     *
     * @param Flux $flux
     */
    public function removeFlux(Flux $flux)
    {
        $flux->removeGroup($this);
        $this->fluxs->removeElement($flux);
    }

    /**
     * Get fluxs
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getFluxs()
    {
        return $this->fluxs;
    }

    /**
     * Add notice
     *
     * @param Notice $notice
     *
     * @return Group
     */
    public function addNotice(Notice $notice)
    {
        $notice->addGroup($this);
        $this->notices[] = $notice;

        return $this;
    }

    /**
     * Remove notice
     *
     * @param Notice $notice
     */
    public function removeNotice(Notice $notice)
    {
        $notice->removeGroup($this);
        $this->notices->removeElement($notice);
    }

    /**
     * Get notices
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getNotices()
    {
        return $this->notices;
    }

    /**
     * Add calendar
     *
     * @param Calendar $calendar
     *
     * @return Group
     */
    public function addCalendar(Calendar $calendar)
    {
        $calendar->addGroup($this);
        $this->calendars[] = $calendar;

        return $this;
    }

    /**
     * Remove calendar
     *
     * @param Calendar $calendar
     */
    public function removeCalendar(Calendar $calendar)
    {
        $calendar->removeGroup($this);
        $this->calendars->removeElement($calendar);
    }

    /**
     * Get calendars
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getCalendars()
    {
        return $this->calendars;
    }

    /**
     * Add blog
     *
     * @param Blog $blog
     *
     * @return Group
     */
    public function addBlog(Blog $blog)
    {
        $blog->addGroup($this);
        $this->blogs[] = $blog;

        return $this;
    }

    /**
     * Remove blog
     *
     * @param Blog $blog
     */
    public function removeBlog(Blog $blog)
    {
        $blog->removeGroup($this);
        $this->blogs->removeElement($blog);
    }

    /**
     * Get blogs
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getBlogs()
    {
        return $this->blogs;
    }

    /**
     * Set description
     *
     * @param string $description
     *
     * @return Group
     */
    public function setDescription($description)
    {
        $this->description = $description;

        return $this;
    }

    /**
     * Get description
     *
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * Set icon
     *
     * @param Icon $icon
     *
     * @return Group
     */
    public function setIcon(Icon $icon = null)
    {
        $this->icon = $icon;

        return $this;
    }

    /**
     * Get icon
     *
     * @return Icon
     */
    public function getIcon()
    {
        return $this->icon;
    }

    /**
     * Set owner
     *
     * @param User $owner
     *
     * @return Group
     */
    public function setOwner(User $owner = null)
    {
        $this->owner = $owner;

        return $this;
    }

    /**
     * Get owner
     *
     * @return User
     */
    public function getOwner()
    {
        return $this->owner;
    }

    /**
     * Set color
     *
     * @param string $color
     *
     * @return Group
     */
    public function setColor($color)
    {
        $this->color = $color;

        return $this;
    }

    /**
     * Get color
     *
     * @return string
     */
    public function getColor()
    {
        return $this->color;
    }

    /**
     * Set pagetemplate
     *
     * @param Page $pagetemplate
     *
     * @return Group
     */
    public function setPagetemplate(Page $pagetemplate = null)
    {
        $this->pagetemplate = $pagetemplate;

        return $this;
    }

    /**
     * Get pagetemplate
     *
     * @return Page
     */
    public function getPagetemplate()
    {
        return $this->pagetemplate;
    }

    /**
     * Add message
     *
     * @param Message $message
     *
     * @return Group
     */
    public function addMessage(Message $message)
    {
        $this->messages[] = $message;

        return $this;
    }

    /**
     * Remove message
     *
     * @param Message $message
     */
    public function removeMessage(Message $message)
    {
        $this->messages->removeElement($message);
    }

    /**
     * Get messages
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getMessages()
    {
        return $this->messages;
    }

    /**
     * Add calendarevent
     *
     * @param Calendarevent $calendarevent
     *
     * @return Group
     */
    public function addCalendarevent(Calendarevent $calendarevent)
    {
        $this->calendarevents[] = $calendarevent;

        return $this;
    }

    /**
     * Remove calendarevent
     *
     * @param Calendarevent $calendarevent
     */
    public function removeCalendarevent(Calendarevent $calendarevent)
    {
        $this->calendarevents->removeElement($calendarevent);
    }

    /**
     * Get calendarevents
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getCalendarevents()
    {
        return $this->calendarevents;
    }

    /**
     * Add project
     *
     * @param Project $project
     *
     * @return Group
     */
    public function addProject(Project $project)
    {
        $project->addGroup($this);
        $this->projects[] = $project;

        return $this;
    }

    /**
     * Remove project
     *
     * @param Project $project
     */
    public function removeProject(Project $project)
    {
        $project->removeGroup($this);
        $this->projects->removeElement($project);
    }

    /**
     * Get projects
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getProjects()
    {
        return $this->projects;
    }


    /**
     * Set fgcancreateproject
     *
     * @param boolean $fgcancreateproject
     *
     * @return Group
     */
    public function setFgcancreateproject($fgcancreateproject)
    {
        $this->fgcancreateproject = $fgcancreateproject;

        return $this;
    }

    /**
     * Get fgcancreateproject
     *
     * @return boolean
     */
    public function getFgcancreateproject()
    {
        return $this->fgcancreateproject;
    }

    /**
     * Add statistic
     *
     * @param Statistic $statistic
     *
     * @return Group
     */
    public function addStatistic(Statistic $statistic)
    {
        $this->statistics[] = $statistic;

        return $this;
    }

    /**
     * Remove statistic
     *
     * @param Statistic $statistic
     */
    public function removeStatistic(Statistic $statistic)
    {
        $this->statistics->removeElement($statistic);
    }

    /**
     * Get statistics
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getStatistics()
    {
        return $this->statistics;
    }

    /**
     * Set email.
     *
     * @param string|null $email
     *
     * @return Group
     */
    public function setEmail($email = null)
    {
        $this->email = $email;

        return $this;
    }

    /**
     * Get email.
     *
     * @return string|null
     */
    public function getEmail()
    {
        return $this->email;
    }

    /**
     * Set invitations
     *
     * @param array $invitations
     *
     * @return Group
     */
    public function setInvitations($invitations)
    {
        $this->invitations = $invitations;

        return $this;
    }

    /**
     * Get invitations
     *
     * @return array
     */
    public function getInvitations()
    {
        return $this->invitations;
    }
}
